// Author:  Prof. Dr. Schmidt <Ulrich.Schmidt@thi.de>
package edu.thi.phga.aparapi_test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.aparapi.device.OpenCLDevice;
import com.aparapi.internal.opencl.OpenCLPlatform;

public class Devices {
  public static OpenCLDevice selectDevice() {
    // Informationen über alle verfügbaren OpenCL-Implementierungen ausgeben

    class Processor {                    // Hilfsklasse
      String name, version, typ;
      OpenCLDevice device;

      Processor(OpenCLPlatform platform, OpenCLDevice device) {
        this.device = device;
        name = platform.getName();
        Matcher m = Pattern.compile("\\d.\\d").matcher(platform.getVersion());
        version = m.find() ? m.group() : "";
        typ = device.getType().name();
      }

      @Override public String toString() {
        return typ + " - OpenCL " + version + " - " + name;
      }
    }

    // erzeuge alle Platform-Device-Kombinationen
    List<Processor> processors = new ArrayList<>();

    for (var platform : OpenCLPlatform.getUncachedOpenCLPlatforms())
      for (var device : platform.getOpenCLDevices())
        processors.add(new Processor(platform, device));

    for (int i = 0; i < processors.size(); i++)
      System.out.println(i + 1 + ") " + processors.get(i));

    var processor = processors.get(App.choice);
    System.out.println("\n" + processor + "\n");

    return processor.device;
  }
}